@@ -10,6 +10,11 @@ CURRENCY_WORD_MAPPING = {
     "es": {"$": "dólar", "€": "euro", "£": "libra"},
     "pt": {"$": "dólar", "€": "euro", "£": "libra"},
     "it": {"$": "dollaro", "€": "euro", "£": "libbra"},
+    "sv": {"$": "dollar", "€": "euro", "£": "pund"},
+    "da": {"$": "dollar", "€": "euro", "£": "pund"},
+    "fi": {"$": "dollari", "€": "euroa", "£": "punta"},
+    "no": {"$": "dollar", "€": "euro", "£": "pund"},
+    "ru": {"$": "доллар", "€": "евро", "£": "фунт"},
 }
 
 INVALID_CHARS = {
@@ -19,6 +24,11 @@ INVALID_CHARS = {
     "es": r"[^a-zA-ZÀ-ÿáéíñúüÁÉÍÑÓÚÜ]",
     "pt": r"[^a-zA-ZÀ-ÿ]",
     "it": r"[^a-zA-ZÀ-ÿ]",
+    "sv": r"[^a-zA-ZÆØÅæøåÉÈÊÓÒÂÔéèêóòâôç]",
+    "da": r"[^a-zA-ZÆØÅæøåÉÈÊÓÒÂÔéèêóòâôç]",
+    "fi": r"[^a-zA-ZÅÄÖåäöŠšŽž]",
+    "no": r"[^a-zA-ZÆØÅæøåÉÈÊÓÒÂÔéèêóòâôç]",
+    "ru": r"[^АаБбВвГгДдЕеЁёЖжЗзИиЙйКкЛлМмНнОоПпСсТтУуФфХхЦцЧчШшЩщЪъЫыЬьЭэЮюЯя]",
 }
 
 PERCENT_WORD = {
@@ -28,8 +38,25 @@ PERCENT_WORD = {
     "es": "por ciento",
     "pt": "por cento",
     "it": "per cento",
+    "sv": "procent",
+    "da": "procent",
+    "fi": "prosenttia",
+    "no": "prosent",
+    "ru": "процентов",
+}
+AND_WORD = {
+    "en": "and",
+    "de": "und",
+    "fr": "et",
+    "es": "y",
+    "pt": "e",
+    "it": "e",
+    "sv": "och",
+    "da": "og",
+    "fi": "ja",
+    "no": "og",
+    "ru": "и",
 }
-AND_WORD = {"en": "and", "de": "und", "fr": "et", "es": "y", "pt": "e", "it": "e"}
 
 METRIC_WORD = {
     "en": {
@@ -74,6 +101,41 @@ METRIC_WORD = {
         "kg": "chilogrammo",
         "cm": "centimetro",
     },
+    "sv": {
+        "km/h": "kilometer i timmen",
+        "mph": "Mil per timme",
+        "m/s": "meter per sekund",
+        "kg": "kilogram",
+        "cm": "centimeter",
+    },
+    "da": {
+        "km/h": "kilometer i timen",
+        "mph": "mil i timen",
+        "m/s": "meter i sekundet",
+        "kg": "kilogram",
+        "cm": "centimeter",
+    },
+    "fi": {
+        "km/h": "kilometriä tunnissa",
+        "mph": "mailia tunnissa",
+        "m/s": "metriä sekunnissa",
+        "kg": "kilogramma",
+        "cm": "senttimetri",
+    },
+    "no": {
+        "km/h": "kilometer i timen",
+        "mph": "miles per time",
+        "m/s": "meter per sekund",
+        "kg": "kilogram",
+        "cm": "centimeter",
+    },
+    "ru": {
+        "km/h": "километров в час",
+        "mph": "миль в час",
+        "m/s": "метры в секунду",
+        "kg": "килограмм",
+        "cm": "сантиметр",
+    },
 }
 
 YEAR_PRECEEDING_WORDS = {
@@ -83,6 +145,11 @@ YEAR_PRECEEDING_WORDS = {
     "es": ["en", "año", "desde"],
     "pt": ["em", "ano", "antes"],
     "it": ["nel", "anno", "prima"],
+    "sv": ["år"],
+    "da": ["år"],
+    "fi": ["vuosi", "vuonna"],
+    "no": ["år", "i"],
+    "ru": ["в"],
 }
 
 # the dict key is regex
@@ -125,10 +192,10 @@ ABBR_WORDS = {
         r"\bNK\b": "Nebenkosten",
     },
     "fr": {
-        r"\bAuj\. ": "aujourd’hui",
+        r"\bAuj\. ": "aujourd'hui",
         r"\bEx.\.": "exemple",
         r"\bQQ\b": "quelque",
-        r"\bQQ’un\b": "quelqu'un",
+        r"\bQQ'un\b": "quelqu'un",
         r"\bS\.": "siècle",
     },
     "es": {
@@ -146,22 +213,28 @@ ABBR_WORDS = {
         r"\bSr\.": "senhor",
         r"\bSra\.": "senhora",
     },
-    "it": {},
 }
 
 ABBR_SEARCH_FLAG = {
-    "en": re.IGNORECASE,
     "de": 0,
-    "fr": re.IGNORECASE,
-    "es": re.IGNORECASE,
-    "pt": re.IGNORECASE,
-    "it": re.IGNORECASE,
 }
 
 
 class BaseFormatter:
     def __init__(self, lang="en"):
-        if lang not in ["en", "de", "fr", "es", "pt", "it"]:
+        if lang not in [
+            "en",
+            "de",
+            "fr",
+            "es",
+            "pt",
+            "it",
+            "ru",
+            "da",
+            "sv",
+            "fi",
+            "no",
+        ]:
             raise Exception("language not implemented")
 
         self.lang = lang
@@ -232,7 +305,7 @@ class TextFormatter(BaseFormatter):
 
     def contains_metric(self, word):
         w = word.lower()
-        keys = METRIC_WORD[self.lang].keys()
+        keys = METRIC_WORD.get(self.lang, {}).keys()
         for k in keys:
             if w.endswith(k):
                 return k
@@ -340,8 +413,8 @@ class TextFormatter(BaseFormatter):
 
         # handle dot and abbr words first
         # so it not confuses with period and illegal dots
-        search_flag = ABBR_SEARCH_FLAG[self.lang]
-        for abbr, repl in ABBR_WORDS[self.lang].items():
+        search_flag = ABBR_SEARCH_FLAG.get(self.lang, re.IGNORECASE)
+        for abbr, repl in ABBR_WORDS.get(self.lang, {}).items():
             if isinstance(repl, tuple):
                 repl_reg, repl_idx, repl_string = repl
             else:
@@ -371,7 +444,8 @@ class TextFormatter(BaseFormatter):
                     int(word) < 2100
                     and int(word) > 1700
                     or idx > 0
-                    and text_iter[idx - 1].lower() in YEAR_PRECEEDING_WORDS[self.lang]
+                    and text_iter[idx - 1].lower()
+                    in YEAR_PRECEEDING_WORDS.get(self.lang, [])
                 )
             ):
                 # this needs further improvements
@@ -529,9 +603,6 @@ class PhoneFormatter(BaseFormatter):
 
         phone_root = self.get_phone_root(phone)
 
-        if self.lang in ["en", "es", "pt"]:
-            return phone_root[0].lower() in ["a", "e", "i", "o", "u"]
-
         if self.lang in ["it"]:
             return phone_root[0].lower() in ["a", "e", "i", "o", "u", "j"]
 
@@ -568,4 +639,5 @@ class PhoneFormatter(BaseFormatter):
                 "Q",
             ]
 
-        return False
+        # make aeiou it default
+        return phone_root[0].lower() in ["a", "e", "i", "o", "u"]
